@@ -19,18 +19,21 @@
 
 #include "kerncompat.h"
 #include <dirent.h>
+#include "kernel-lib/bitops.h"
 #include "kernel-shared/uapi/btrfs.h"
 
-#define BTRFS_SCAN_MOUNTED	(1ULL << 0)
-#define BTRFS_SCAN_LBLKID	(1ULL << 1)
+enum {
+	BTRFS_SCAN_MOUNTED	= (1ULL << 0),
+	BTRFS_SCAN_LBLKID	= (1ULL << 1)
+};
 
-#define BTRFS_UPDATE_KERNEL	1
-
-#define BTRFS_ARG_UNKNOWN	0
-#define BTRFS_ARG_MNTPOINT	1
-#define BTRFS_ARG_UUID		2
-#define BTRFS_ARG_BLKDEV	3
-#define BTRFS_ARG_REG		4
+enum {
+	BTRFS_ARG_UNKNOWN,
+	BTRFS_ARG_MNTPOINT,
+	BTRFS_ARG_UUID,
+	BTRFS_ARG_BLKDEV,
+	BTRFS_ARG_REG,
+};
 
 #define SEEN_FSID_HASH_SIZE 256
 
@@ -40,11 +43,11 @@ struct btrfs_trans_handle;
 struct seen_fsid {
 	u8 fsid[BTRFS_FSID_SIZE];
 	struct seen_fsid *next;
-	DIR *dirstream;
 	int fd;
 };
 
 int btrfs_scan_devices(int verbose);
+int btrfs_scan_argv_devices(int dev_optind, int argc, char **argv);
 int btrfs_register_one_device(const char *fname);
 int btrfs_register_all_devices(void);
 int btrfs_add_to_fsid(struct btrfs_trans_handle *trans,
@@ -54,8 +57,7 @@ int btrfs_add_to_fsid(struct btrfs_trans_handle *trans,
 int btrfs_device_already_in_root(struct btrfs_root *root, int fd,
 				 int super_offset);
 int is_seen_fsid(u8 *fsid, struct seen_fsid *seen_fsid_hash[]);
-int add_seen_fsid(u8 *fsid, struct seen_fsid *seen_fsid_hash[],
-		int fd, DIR *dirstream);
+int add_seen_fsid(u8 *fsid, struct seen_fsid *seen_fsid_hash[], int fd);
 void free_seen_fsid(struct seen_fsid *seen_fsid_hash[]);
 int test_uuid_unique(const char *uuid_str);
 
