@@ -10,6 +10,7 @@ check_global_prereq mke2fs
 
 setup_root_helper
 prepare_test_dev
+check_kernel_support_acl
 
 # simple wrapper for a convert test
 # $1: btrfs features, argument to -O
@@ -37,6 +38,10 @@ do_test() {
 
 	convert_test_do_convert "$features" "$nodesize"
 
+	if ! convert_can_mount "$features"; then
+		return 0
+	fi
+
 	run_check_mount_test_dev
 	convert_test_post_check_checksums "$CHECKSUMTMP"
 
@@ -63,10 +68,8 @@ do_test() {
 
 # Iterate over defaults and options that are not tied to hardware capabilities
 # or number of devices
-for feature in '' 'block-group-tree' ; do
+for feature in '' 'block-group-tree' 'raid-stripe-tree' ; do
 	do_test "$feature" "ext4 4k nodesize" 4096 mke2fs -t ext4 -b 4096
-	do_test "$feature" "ext4 8k nodesize" 8192 mke2fs -t ext4 -b 4096
 	do_test "$feature" "ext4 16k nodesize" 16384 mke2fs -t ext4 -b 4096
-	do_test "$feature" "ext4 32k nodesize" 32768 mke2fs -t ext4 -b 4096
 	do_test "$feature" "ext4 64k nodesize" 65536 mke2fs -t ext4 -b 4096
 done
